from shekar.base import BaseTransform
from shekar.tokenization import AlbertTokenizer
from shekar.hub import Hub
from pathlib import Path
import onnxruntime
import numpy as np


class AlbertNER(BaseTransform):
    def __init__(self, model_path: str | Path = None):
        super().__init__()
        resource_name = "albert_persian_ner_q8.onnx"
        if model_path is None or not Path(model_path).exists():
            model_path = Hub.get_resource(file_name=resource_name)

        self.session = onnxruntime.InferenceSession(model_path)
        self.tokenizer = AlbertTokenizer()

        self.id2tag = {
            0: "B-DAT",
            1: "B-EVE",
            2: "B-LOC",
            3: "B-ORG",
            4: "B-PER",
            5: "I-DAT",
            6: "I-EVE",
            7: "I-LOC",
            8: "I-ORG",
            9: "I-PER",
            10: "O",
        }

    def _aggregate_entities(self, tokens, predicted_tag_ids):
        entities = []
        current_entity = ""
        current_label = None

        for token, tag_id in zip(tokens, predicted_tag_ids):
            label = self.id2tag[tag_id]

            if token in ["[CLS]", "[SEP]"]:
                continue

            is_new_word = token.startswith("▁")
            clean_token = token.lstrip("▁")

            if clean_token == "‌":
                if current_entity:
                    current_entity = current_entity.rstrip() + "\u200c"
                continue

            if label.startswith("B-"):
                if current_entity:
                    entities.append((current_entity.strip(), current_label))
                current_entity = clean_token
                current_label = label[2:]
            elif label.startswith("I-") and current_label == label[2:]:
                if current_entity.endswith("\u200c"):
                    current_entity += clean_token
                elif is_new_word:
                    current_entity += " " + clean_token
                else:
                    current_entity += clean_token
            else:
                if current_entity:
                    entities.append((current_entity.strip(), current_label))
                    current_entity = ""
                    current_label = None

        if current_entity:
            entities.append((current_entity.strip(), current_label))

        return entities

    def transform(self, X: str) -> list:
        inputs = self.tokenizer.fit_transform(X)
        inputs.pop("token_type_ids", None)
        outputs = self.session.run(None, inputs)
        logits = outputs[0]
        predicted_tag_ids = np.argmax(logits, axis=-1)[0]
        tokens = [
            self.tokenizer.tokenizer.id_to_token(id) for id in inputs["input_ids"][0]
        ]
        entities = self._aggregate_entities(tokens, predicted_tag_ids)
        return entities



import pytest
import os
import tempfile
import hashlib
from pathlib import Path
from unittest import mock
from shekar.hub import Hub, MODEL_HASHES, TqdmUpTo


class TestHub:
    def test_compute_sha256_hash(self):
        # Create a temporary file with known content
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            tmp.write(b"test content")
            tmp_path = tmp.name

        try:
            # Calculate expected hash
            expected_hash = hashlib.sha256(b"test content").hexdigest()
            # Test the method
            actual_hash = Hub.compute_sha256_hash(tmp_path)
            assert actual_hash == expected_hash
        finally:
            # Clean up
            os.unlink(tmp_path)

    def test_compute_sha256_hash_with_path_object(self):
        # Test with a Path object instead of a string
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            tmp.write(b"test content with path")
            tmp_path = Path(tmp.name)

        try:
            expected_hash = hashlib.sha256(b"test content with path").hexdigest()
            actual_hash = Hub.compute_sha256_hash(tmp_path)
            assert actual_hash == expected_hash
        finally:
            os.unlink(tmp_path)

    def test_get_resource_file_not_recognized(self):
        # Test for an unrecognized file
        with pytest.raises(ValueError) as excinfo:
            Hub.get_resource("nonexistent_file.txt")
        assert "File nonexistent_file.txt is not recognized" in str(excinfo.value)

    @mock.patch("shekar.hub.Hub.download_file")
    @mock.patch("shekar.hub.Hub.compute_sha256_hash")
    @mock.patch("pathlib.Path.exists")
    @mock.patch("pathlib.Path.mkdir")
    def test_get_resource_download_success(
        self, mock_mkdir, mock_exists, mock_hash, mock_download
    ):
        # Setup mocks
        mock_exists.return_value = False
        mock_download.return_value = True
        mock_hash.return_value = MODEL_HASHES["albert_persian_tokenizer.json"]

        # Call the method
        result = Hub.get_resource("albert_persian_tokenizer.json")

        # Assertions
        assert mock_mkdir.called
        assert mock_download.called
        assert isinstance(result, Path)
        assert result.name == "albert_persian_tokenizer.json"

    @mock.patch("shekar.hub.Hub.download_file")
    @mock.patch("pathlib.Path.exists")
    @mock.patch("pathlib.Path.unlink")
    @mock.patch("pathlib.Path.mkdir")
    def test_get_resource_download_failure(
        self, mock_mkdir, mock_unlink, mock_exists, mock_download
    ):
        # Setup mocks
        mock_exists.return_value = False
        mock_download.return_value = False

        # Call the method and check for exception
        with pytest.raises(FileNotFoundError) as excinfo:
            Hub.get_resource("albert_persian_tokenizer.json")

        assert "Failed to download" in str(excinfo.value)
        assert mock_unlink.called

    @mock.patch("shekar.hub.Hub.compute_sha256_hash")
    @mock.patch("pathlib.Path.exists")
    @mock.patch("pathlib.Path.unlink")
    @mock.patch("pathlib.Path.mkdir")
    def test_get_resource_hash_mismatch(
        self, mock_mkdir, mock_unlink, mock_exists, mock_hash
    ):
        # Setup mocks
        mock_exists.return_value = True
        mock_hash.return_value = "wrong_hash_value"

        # Call the method and check for exception
        with pytest.raises(ValueError) as excinfo:
            Hub.get_resource("albert_persian_tokenizer.json")

        assert "Hash mismatch" in str(excinfo.value)
        assert mock_unlink.called

    @mock.patch("urllib.request.urlretrieve")
    def test_download_file_success(self, mock_urlretrieve):
        # Setup
        url = "https://example.com/file.txt"
        dest_path = Path(tempfile.gettempdir()) / "file.txt"

        # Test
        result = Hub.download_file(url, dest_path)

        # Assertions
        assert result is True
        mock_urlretrieve.assert_called_once()

    @mock.patch("urllib.request.urlretrieve")
    def test_download_file_failure(self, mock_urlretrieve):
        # Setup
        mock_urlretrieve.side_effect = Exception("Download failed")
        url = "https://example.com/file.txt"
        dest_path = Path(tempfile.gettempdir()) / "file.txt"

        # Test
        result = Hub.download_file(url, dest_path)

        # Assertions
        assert result is False

    def test_tqdm_up_to(self):
        # Create a TqdmUpTo instance
        with mock.patch("tqdm.tqdm.update") as mock_update:
            t = TqdmUpTo(total=100)

            # Test with tsize=None
            t.update_to(b=1, bsize=10)
            mock_update.assert_called_with(10)

            # Test with tsize specified
            t.update_to(b=2, bsize=10, tsize=200)
            assert t.total == 200
            mock_update.assert_called_with(20)
